@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -112,13 +112,13 @@ class CommandManager(object):
             checks and adding it to registered commands.
             """
             if not util.RE_COMMAND_NAME.match(epoint.name):
-                LOG().error('invalid command name: %s, ignoring', epoint.name)
+                LOG().error('Invalid command name: %s, ignoring.', epoint.name)
                 return
             if epoint.name in self._commands:
-                LOG().warn('command "%s" already registered, ignoring',
+                LOG().warn('Command "%s" already registered, ignoring.',
                         epoint.name)
             else:
-                LOG().debug('found command "%s"', epoint.name)
+                LOG().debug('Found registered command "%s".', epoint.name)
                 self._commands[epoint.name] = epoint
 
         for entry_point in pkg_resources.iter_entry_points(self._namespace):
@@ -144,7 +144,7 @@ class CommandManager(object):
             raise errors.LmiCommandInvalidName(
                     cmd_class.__module__, cmd_class.__class__.__name__, name)
         if name in self._commands:
-            LOG().warn('command "%s" already managed, overwriting with "%s:%s"',
+            LOG().warn('Command "%s" already managed, overwriting with "%s:%s".',
                     name, cmd_class.__module__, cmd_class.__name__)
         self._commands[name] = _CustomCommandWrapper(name, cmd_class)
 
@@ -159,6 +159,10 @@ class CommandManager(object):
             return self._commands[cmd_name].load()
         except KeyError:
             raise errors.LmiCommandNotFound(cmd_name)
+        except ImportError as err:
+            LOG().debug('Failed to import command "%s".', cmd_name, exc_info=err)
+            raise errors.LmiCommandImportError(
+                    cmd_name, self._commands[cmd_name].module_name, err)
 
     def reload_commands(self, keep_custom=True):
         """
