@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -84,9 +84,9 @@ Handling hosts:
 
 import docopt
 
+from lmi.scripts._metacommand import exit
 from lmi.scripts._metacommand import util
 from lmi.scripts._metacommand import Interactive
-from lmi.scripts._metacommand.exit import Exit
 from lmi.scripts.common import get_logger
 from lmi.scripts.common import errors
 from lmi.scripts.common.command import base
@@ -107,11 +107,15 @@ class TopLevelCommand(base.LmiBaseCommand):
     def child_commands(cls):
         return []
 
+    @classmethod
+    def is_end_point(cls):
+        return False
+
     def __init__(self, app, cmd_name='lmi'):
         base.LmiBaseCommand.__init__(self, app, cmd_name)
 
-    def get_usage(self, _proper=False):
-        return USAGE_STRING % { 'cmd' : self.cmd_full_name }
+    def get_usage(self, proper=False):
+        return USAGE_STRING[1:] % { 'cmd' : " ".join(self.cmd_name_parts) }
 
     def run_subcommand(self, cmd_name, args):
         """
@@ -123,12 +127,16 @@ class TopLevelCommand(base.LmiBaseCommand):
 
     def start_interactive_mode(self):
         """ Run the command line loop of interactive application. """
-        self.app.command_manager.add_command("exit", Exit)
-        iapp = Interactive(self.app, self.cmd_name + "> ")
-        try:
-            return iapp.cmdloop()
-        except errors.LmiTerminate as err:
-            return err.args[0]
+        self.app.command_manager.add_command("exit", exit.Exit)
+        iapp = Interactive(self)
+        while True:
+            try:
+                return iapp.cmdloop()
+            except errors.LmiTerminate as err:
+                return err.args[0]
+            except KeyboardInterrupt as err:
+                LOG().debug('%s: %s', err.__class__.__name__, str(err))
+                self.app.stdout.write('\n')
 
     def run(self, args):
         """
@@ -144,21 +152,29 @@ class TopLevelCommand(base.LmiBaseCommand):
                     version=util.get_version(), help=False, options_first=True)
         except docopt.DocoptLanguageError as exc:
             self.app.stderr.write("%s\n" % str(exc))
-            return 1
+            return exit.EXIT_CODE_FAILURE
         if options.pop('--help', False):
             self.app.stdout.write(self.get_usage())
             self.app.stdout.write("\nCommands:\n")
             self.app.stdout.write("    %s\n" % " ".join(
                 n for n in sorted(self.app.command_manager)))
-            return 0
+            return exit.EXIT_CODE_SUCCESS
         if options.pop('--version', False):
             self.app.print_version()
-            return 0
+            return exit.EXIT_CODE_SUCCESS
         self.app.setup(options)
         if options['<command>'] is None:
             return self.start_interactive_mode()
-        else:
-            LOG().debug('running command "%s"', options['<command>'])
-            return self.run_subcommand(
-                    options['<command>'], options['<args>'])
+
+        try:
+            LOG().debug('Running command "%s".', options['<command>'])
+            return self.run_subcommand(options['<command>'], options['<args>'])
+        except docopt.DocoptExit as err:
+            if '--help' in args:
+                cmd_args = options['<args>']
+                cmd_args = cmd_args[:cmd_args.index('--help')]
+                return self.run_subcommand(
+                        'help', [options['<command>']] + cmd_args)
+            raise
+
 
