@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -34,24 +34,38 @@ Meta-command utility module.
 import logging
 import logging.config
 import pkg_resources
+import platform
 import re
 import sys
 import urlparse
 
 from lmi.scripts.common import Configuration
 from lmi.scripts.common import get_logger
+from lmi.scripts.common import lmi_logging
 
 PYTHON_EGG_NAME = "openlmi-scripts"
 
 RE_NETLOC = re.compile(r'^((?P<username>[^:@]+)(:(?P<password>[^@]+))?@)?'
         r'(?P<hostname>[^:]+)(:(?P<port>\d+))?$')
 
+VERBOSITY_2_LOG_LEVEL = {
+    Configuration.OUTPUT_SILENT  : logging.ERROR,
+    Configuration.OUTPUT_WARNING : logging.WARNING,
+    Configuration.OUTPUT_INFO    : logging.INFO,
+    Configuration.OUTPUT_DEBUG   : logging.DEBUG,
+}
+
 DEFAULT_LOGGING_CONFIG = {
     'version' : 1,
     'disable_existing_loggers': True,
     'formatters' : {
         'console' : {
-            'format': Configuration.default_options()['ConsoleFormat'],
+            "()" : "lmi.scripts.common.lmi_logging.LevelDispatchingFormatter",
+            "formatters" : {
+                logging.INFO :
+                    Configuration.default_options()['ConsoleInfoFormat']
+            },
+            'default': Configuration.default_options()['ConsoleFormat'],
             'datefmt' : '%Y-%m-%d %H:%M:%S'
         },
         'file' : {
@@ -65,12 +79,24 @@ DEFAULT_LOGGING_CONFIG = {
             'level' : logging.ERROR,
             'formatter': 'console',
         },
+        'console_shell': {
+            'class' : "logging.StreamHandler",
+            'level' : logging.CRITICAL,
+            'formatter': 'console',
+        },
         'file' : {
             'class' : "logging.FileHandler",
             'level' : Configuration.default_options()['Level'].upper(),
             'formatter': 'file',
         }
     },
+    'loggers' : {
+        'lmi.shell' : {
+            'handlers' : ['console_shell'],
+            'level' : logging.DEBUG,
+            'propagate' : False
+        }
+    },
     'root' : {
         'level' : logging.DEBUG,
         'handlers' : ['console']
@@ -99,9 +125,10 @@ def setup_logging(app_config, stderr=sys.stderr):
             cfg['handlers']['file']['level'] = \
                     getattr(logging, app_config.logging_level.upper())
         except KeyError:
-            LOG().error('unsupported logging level: "%s"',
+            LOG().error('Unsupported logging level: "%s".',
                     app_config.logging_level)
         cfg['root']['handlers'].append('file')
+        cfg['loggers']['lmi.shell']['handlers'].append('file')
     else:
         del cfg['formatters']['file']
         del cfg['handlers']['file']
@@ -110,18 +137,33 @@ def setup_logging(app_config, stderr=sys.stderr):
         # Set up logging to console
         if stderr is not sys.stderr:
             cfg['handlers']['console']['stream'] = stderr
-        cfg['handlers']['console']['level'] = {
-                Configuration.OUTPUT_SILENT  : logging.CRITICAL,
-                Configuration.OUTPUT_WARNING : logging.WARNING,
-                Configuration.OUTPUT_INFO    : logging.INFO,
-                Configuration.OUTPUT_DEBUG   : logging.DEBUG,
-            }.get(app_config.verbosity)
-        cfg['formatters']['console']['format'] = app_config.get_safe(
+        cfg['handlers']['console']['level'] = \
+                VERBOSITY_2_LOG_LEVEL[app_config.verbosity]
+
+        # make the verbosity of lmi shell one level less
+        lmi_shell_verbosity = app_config.verbosity
+        if (   app_config.verbosity < app_config.OUTPUT_DEBUG
+           and app_config.verbosity > app_config.OUTPUT_SILENT):
+            lmi_shell_verbosity -= 1
+        cfg['handlers']['console_shell']['level'] = \
+                VERBOSITY_2_LOG_LEVEL[lmi_shell_verbosity]
+
+        # use ConsoleInfoFormat for INFO and less severe levels
+        cfg['formatters']['console']['formatters'] = {
+            logging.INFO :
+                    app_config.get_safe('Log', 'ConsoleInfoFormat', raw=True)
+        }
+        # use ConsoleFormat for any other level
+        cfg['formatters']['console']['default'] = app_config.get_safe(
                 'Log', 'ConsoleFormat', raw=True)
     else:
-        del cfg['handlers']['console']
         cfg['root']['handlers'].remove('console')
+        cfg['loggers']['lmi.shell']['handlers'].remove('console_shell')
+        del cfg['handlers']['console']
+        del cfg['handlers']['console_shell']
 
+    use_colors = platform.system() != 'Windows' and stderr.isatty()
+    lmi_logging.setup_logger(use_colors = use_colors)
     logging.config.dictConfig(cfg)
 
 def get_version(egg_name=PYTHON_EGG_NAME):
