@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -39,6 +39,14 @@ import re
 # containing non-white-spaces
 RE_LSPACES = re.compile(r'\A(\s*$.)*', re.DOTALL | re.MULTILINE)
 
+#: Default formatting options overriden by options passed onc ommand-line and
+#: set in configuration file.
+DEFAULT_FORMATTER_OPTIONS = {
+    'no_headings'    : False,
+    'padding'        : 0,
+    'human_friendly' : False,
+}
+
 class LmiBaseCommand(object):
     """
     Abstract base class for all commands handling command line arguemtns.
@@ -136,45 +144,68 @@ class LmiBaseCommand(object):
         return self._cmd_name
 
     @property
-    def cmd_full_name(self):
+    def cmd_name_parts(self):
         """
-        Name of this subcommand with all prior commands included.
-        It's the sequence of commands as given on command line up to this
-        subcommand without any options present. In interactive mode
-        this won't contain the name of binary (``sys.argv[0]``).
-
-        :returns:  Concatenation of all preceding commands with
-            :py:attr:`cmd_name`.
-        :rtype: string
-        """
-        return ' '.join(self.cmd_name_args)
-
-    @property
-    def cmd_name_args(self):
-        """
-        The same as :py:attr:`cmd_full_name`, except the result is a list of
-        subcommands.
-
-        :returns: List of command strings as given on command line up to this
-            command.
-        :rtype: list
-        """
-        if self.parent is not None:
-            return self.parent.cmd_name_args + [self.cmd_name]
-        return [self._cmd_name]
-
-    @property
-    def docopt_cmd_name_args(self):
-        """
-        Arguments array for docopt parser. Similar to
-        :py:meth:`LmiBaseCommand.cmd_name_args` except for the leading binary
-        name, which is omitted here.
+        Convenience property calling :py:meth:`get_cmd_name_parts` to obtain
+        command path as a list of all preceding command names.
 
         :rtype: list
         """
-        if self.app.interactive_mode:
-            return self.cmd_name_args
-        return self.cmd_name_args[1:]
+        return self.get_cmd_name_parts()
+
+    @property
+    def format_options(self):
+        """
+        Compose formatting options. Parent commands are queried for defaults. If
+        command has no parent, default options will be taken from
+        :py:attr:`DEFAULT_FORMATTER_OPTIONS` which are overriden by config
+        settings.
+
+        :returns: Arguments passed to formatter factory when formatter is
+            for current command is constructed.
+        :rtype: dictionary
+        """
+        if self.parent is None:
+            options = DEFAULT_FORMATTER_OPTIONS.copy()
+            options['no_headings'] = self.app.config.no_headings
+            options['human_friendly'] = self.app.config.human_friendly
+        else:
+            options = self.parent.format_options
+        return options
+
+    def get_cmd_name_parts(self, all_parts=False, demand_own_usage=True,
+            for_docopt=False):
+        """
+        Get name of this command as a list composed of names of all preceding
+        commands since the top level one. When in interactive mode, only
+        commands following the active one will be present.
+
+        :param boolean full: Take no heed to the active command or interactive
+            mode. Return all command names since top level node inclusive. This
+            is overriden with *for_docopt* flag.
+        :param boolean demand_own_usage: Wether to continue the upward
+            traversal through command hieararchy past the active command until
+            the command with its own usage is found. This is the default behaviour.
+        :param boolean for_docopt: Docopt parser needs to be given arguments list
+            without the first item compared to command names in usage string
+            it receives. Thus this option causes skipping the first item that would
+            be otherwise included.
+        :returns: Command path. Returned list will always contain at least the
+            name of this command.
+        :rtype: list
+        """
+        parts = [self.cmd_name]
+        cmd = self
+        own_usage = cmd.has_own_usage()
+        while (   cmd.parent is not None
+              and (all_parts or self.app.active_command not in (cmd, cmd.parent))
+              or  (demand_own_usage and not own_usage)):
+            cmd = cmd.parent
+            parts.append(cmd.cmd_name)
+            own_usage = own_usage or cmd.has_own_usage()
+        if for_docopt and parts:
+            parts.pop()
+        return list(reversed(parts))
 
     def get_usage(self, proper=False):
         """
@@ -196,14 +227,23 @@ class LmiBaseCommand(object):
             while not cmd.has_own_usage() and cmd.parent is not None:
                 cmd = cmd.parent
             if cmd.__doc__ is None:
-                docstr = "Usage: %s\n" % self.cmd_full_name
+                docstr = "Usage: %s\n" % " ".join(self.cmd_name_parts)
             else:
                 docstr = ( ( cmd.__doc__.rstrip()
-                           % {'cmd' : cmd.cmd_full_name }
-                           ) + "\n")
+                           % {'cmd' : " ".join(cmd.cmd_name_parts)}
+                           ))
+
                 match = RE_LSPACES.match(docstr)
-                if match:   # strip leading spaces
+                if match:   # strip leading newlines
                     docstr = docstr[match.end(0):]
+
+                match = re.match(r'^ +', docstr)
+                if match:   # unindent help message
+                    re_lspaces = re.compile(r'^ {%s}' % match.end(0))
+                    docstr = "\n".join(re_lspaces.sub('', l)
+                                for l in docstr.splitlines())
+                docstr += "\n"
+
         else:
             # generate usage string from what is known, applies to nodes
             # without own usage
@@ -212,8 +252,8 @@ class LmiBaseCommand(object):
                 hlp.append(self.get_description())
                 hlp.append("")
             hlp.append("Usage:")
-            hlp.append("    %s (--help | <command> [<args> ...])"
-                    % self.cmd_full_name)
+            hlp.append("    %s <command> [<args> ...]"
+                    % " ".join(self.cmd_name_parts))
             hlp.append("")
             hlp.append("Commands:")
             cmd_max_len = max(len(c) for c in self.child_commands())
@@ -221,6 +261,7 @@ class LmiBaseCommand(object):
                 hlp.append(("  %%-%ds %%s" % cmd_max_len)
                         % (name, cmd.get_description()))
             docstr = "\n".join(hlp) + "\n"
+
         return docstr
 
     @abc.abstractmethod
