@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -84,14 +84,14 @@ def options_dict2kwargs(options):
                 break
         else:
             raise errors.LmiError(
-                    'failed to convert argument "%s" to function option' %
+                    'Failed to convert argument "%s" to function option.' %
                     name)
         if new_name == '--':
             continue    # ignore double dash
         new_name = opt_name_sanitize(new_name)
         if new_name in kwargs:
-            raise errors.LmiError('option clash for "%s" and "%s", which both'
-                ' translate to "%s"' % (name, orig_names[new_name], new_name))
+            raise errors.LmiError('Option clash for "%s" and "%s", which both'
+                ' translate to "%s".' % (name, orig_names[new_name], new_name))
         kwargs[new_name] = value
         orig_names[new_name] = name
     return kwargs
@@ -176,7 +176,7 @@ class LmiEndPointCommand(base.LmiBaseCommand):
         :returns: Exit code of application.
         :rtype: integer
         """
-        self.execute(*args, **kwargs)
+        return self.execute(*args, **kwargs)
 
     @property
     def formatter(self):
@@ -186,8 +186,13 @@ class LmiEndPointCommand(base.LmiBaseCommand):
         :rtype: :py:class:`~lmi.scripts.common.formatter.Formatter`
         """
         if self._formatter is None:
-            self._formatter = self.formatter_factory()(
-                    self.app.stdout, no_headings=self.app.config.no_headings)
+            opts = self.format_options
+            factory = self.formatter_factory()
+            argspec = inspect.getargspec(
+                    factory.__init__ if type(factory) is type else factory)
+            if not argspec.keywords:
+                kwargs = {k: v for k, v in opts.items() if k in argspec.args}
+            self._formatter = factory(self.app.stdout, **kwargs)
         return self._formatter
 
     def _make_end_point_args(self, options):
@@ -214,8 +219,9 @@ class LmiEndPointCommand(base.LmiBaseCommand):
             # otherwise we need to remove any unhandled
             for opt_name in kwargs:
                 if opt_name not in argspec.args[pos_args_count:]:
-                    LOG().debug('option "%s" not handled in function "%s",'
-                        ' ignoring', opt_name, self.cmd_name)
+                    if opt_name not in self.cmd_name_parts:
+                        LOG().debug('Option "%s" not handled in function "%s",'
+                            ' ignoring.', opt_name, self.cmd_name)
                     to_remove.append(opt_name)
         for opt_name in to_remove:
             # remove options unhandled by function
@@ -258,7 +264,7 @@ class LmiEndPointCommand(base.LmiBaseCommand):
 
         .. _docopt: http://docopt.org/
         """
-        full_args = self.cmd_name_args[1:] + args
+        full_args = self.get_cmd_name_parts(for_docopt=True) + args
         options = docopt(self.get_usage(), full_args, help=False)
         self._preprocess_options(options)
 
@@ -271,8 +277,8 @@ class LmiEndPointCommand(base.LmiBaseCommand):
                 try:
                     del options[scn]
                 except KeyError:
-                    LOG().warn('usage string of "%s.%s" command does not'
-                            ' contain registered command "%s" command',
+                    LOG().warn('Usage string of "%s.%s" command does not'
+                            ' contain registered command "%s" command.',
                             cmd.__module__, cmd.__class__.__name__, scn)
         # remove also the root command name from options
         if cmd is not None and cmd.cmd_name in options:
