@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -55,6 +55,8 @@ RE_CALLABLE = re.compile(
 RE_ARRAY_SUFFIX = re.compile(r'^(?:[a-z_]+[a-z0-9_]*)?$', re.IGNORECASE)
 RE_OPTION = re.compile(r'^-+(?P<name>[^-+].*)$')
 
+FORMAT_OPTIONS = ('no_headings', 'human_friendly')
+
 LOG = get_logger(__name__)
 
 def _handle_usage(name, dcl):
@@ -155,7 +157,7 @@ def _handle_callable(name, bases, dcl):
                         [match.group('func')], 0),
                         match.group('func'))
             except (ImportError, AttributeError):
-                raise errors.LmiCommandImportFailed(
+                raise errors.LmiImportCallableFailed(
                         dcl['__module__'], name, func)
     except KeyError:
         raise errors.LmiCommandMissingCallable(dcl['__module__'], name)
@@ -211,7 +213,7 @@ def _make_render_with_properties(properties, target_formatter_lister=False):
         if isinstance(prop, basestring):
             prop_name = prop
             if not prop in inst.properties():
-                LOG().warn('property "%s" not present in instance of "%s"',
+                LOG().warn('Property "%s" not present in instance of "%s".',
                         prop, inst.path)
                 value = "UNKNOWN"
             else:
@@ -227,12 +229,8 @@ def _make_render_with_properties(properties, target_formatter_lister=False):
                 else:
                     value = getattr(inst, prop[1])
             except Exception as exc:
-                if Configuration.get_instance().trace:
-                    LOG().exception('failed to render property "%s"',
-                            prop[0])
-                else:
-                    LOG().error('failed to render property "%s": %s',
-                            prop[0], exc)
+                LOG().exception('Failed to render property "%s": %s',
+                        prop[0], exc)
                 value = "ERROR"
         if value is None:
             value = ''
@@ -402,10 +400,10 @@ def _handle_opt_preprocess(name, dcl):
                             if cond(name, value))
                 for name in to_rename:
                     new_name = transform(name)
-                    LOG().debug('renaming option "%s" to "%s"', name, new_name)
+                    LOG().debug('Renaming option "%s" to "%s".', name, new_name)
                     if new_name in options:
                         LOG().warn(
-                                'existing option named "%s" replaced with "%s"',
+                            'Existing option named "%s" replaced with "%s".',
                                 new_name, name)
                     options[new_name] = options.pop(name)
 
@@ -443,6 +441,45 @@ def _handle_fallback_command(name, bases, dcl):
             fallback.has_own_usage = lambda cls: True
         dcl['fallback_command'] = staticmethod(lambda: fallback)
 
+def _handle_format_options(name, bases, dcl):
+    """
+    Process any ``FMT_*`` properties. This overrides ``format_options``
+    property which returns dictionary of arguments passed to formatter factory.
+
+    These properties are removed from class' dictionary.
+    """
+    format_options = {}
+    for key, value in dcl.items():
+        if key.startswith("FMT_"):
+            opt_name = key[4:].lower()
+            if opt_name not in FORMAT_OPTIONS:
+                raise errors.LmiCommandInvalidProperty(
+                        dcl['__module__'], name,
+                        'formatting option "%s" is not supported' %
+                        opt_name)
+            if (   opt_name in ('no_headings', 'human_friendly')
+               and not isinstance(value, bool)):
+                raise errors.LmiCommandInvalidProperty(
+                        dcl['__module__'], name,
+                        '"%s" property must be a boolean')
+            format_options[opt_name] = value
+
+    if format_options:
+        def _new_format_options(self):
+            """ :returns: Dictionary of options for formatter object. """
+            basecls = [b for b in bases if issubclass(b, base.LmiBaseCommand)][0]
+            opts = basecls.format_options.fget(self)
+            opts.update(format_options)
+            return opts
+        if 'format_options' in dcl:
+            raise errors.LmiCommandError(dcl['__module__'], name,
+                    'can not define both FMT_ options and "format_options" in'
+                    ' the same class, choose just one of them')
+        dcl['format_options'] = property(_new_format_options)
+
+    for key in format_options:
+        dcl.pop('FMT_' + key.upper())
+
 class EndPointCommandMetaClass(abc.ABCMeta):
     """
     End point command does not have any subcommands. It's a leaf of
@@ -457,12 +494,19 @@ class EndPointCommandMetaClass(abc.ABCMeta):
         ``ARG_ARRAY_SUFFIX`` : ``str``
             Suffix added to argument names containing array of values.
             Optional property.
+        ``FMT_NO_HEADINGS`` : ``bool``
+            Allows to force printing of table headers on and off for
+            this command. Default is to print them.
+        ``FMT_HUMAN_FRIENDLY`` : ``bool``
+            Tells formatter to make the output more human friendly. The result
+            is dependent on the type of formatter used.
     """
 
     def __new__(mcs, name, bases, dcl):
         _handle_usage(name, dcl)
         _handle_callable(name, bases, dcl)
         _handle_opt_preprocess(name, dcl)
+        _handle_format_options(name, bases, dcl)
 
         cls = super(EndPointCommandMetaClass, mcs).__new__(
                 mcs, name, bases, dcl)
@@ -506,7 +550,18 @@ class ListerMetaClass(SessionCommandMetaClass):
     properties:
 
         ``COLUMNS`` : ``tuple``
-            List of column names. Optional property.
+            List of column names. Optional property. There are special values
+            such as:
+
+                ``None`` or omitted
+                    Associated function provides column names in a first row of
+                    returned list or generator.
+
+                empty list, empty tuple or ``False``
+                    They mean that no headers shall be printed. It is simalar
+                    to using ``FMT_NO_HEADINGS = True``. But in this case all
+                    the rows returned from associated functions are treated as
+                    data.
     """
 
     def __new__(mcs, name, bases, dcl):
@@ -515,7 +570,10 @@ class ListerMetaClass(SessionCommandMetaClass):
             if not isinstance(cols, (list, tuple)):
                 raise errors.LmiCommandInvalidProperty(dcl['__module__'], name,
                         'COLUMNS class property must be either list or tuple')
-            if not all(isinstance(c, basestring) for c in cols):
+            if len(cols) < 1 or cols is False:
+                dcl['FMT_NO_HEADINGS'] = True
+                cols = tuple()
+            elif not all(isinstance(c, basestring) for c in cols):
                 raise errors.LmiCommandInvalidProperty(dcl['__module__'], name,
                         'COLUMNS must contain just column names as strings')
             def _new_get_columns(_cls):
@@ -584,7 +642,7 @@ class CheckResultMetaClass(SessionCommandMetaClass):
                         result = result.rval
                     passed = expect(options, result)
                     if not passed:
-                        LOG().info('got unexpected result "%s"')
+                        LOG().info('Got unexpected result "%s".')
                     return passed
             else:
                 def _new_expect(_self, _options, result):
@@ -593,7 +651,7 @@ class CheckResultMetaClass(SessionCommandMetaClass):
                         result = result.rval
                     passed = expect == result
                     if not passed:
-                        LOG().info('expected "%s", got "%s"', expect, result)
+                        LOG().info('Expected "%s", got "%s".', expect, result)
                         return (False, '%s != %s' % (expect, result))
                     return passed
                 _new_expect.expected = expect
@@ -618,6 +676,9 @@ class MultiplexerMetaClass(abc.ABCMeta):
         ``FALLBACK_COMMAND`` : :py:class:`~.endpoint.LmiEndPointCommand`
             Command factory to use in case that no command is passed on command
             line.
+
+    Formatting options (starting with ``FMT_`` are also accepted, and may used
+    to set defaults for all subcommands.
     """
 
     def __new__(mcs, name, bases, dcl):
@@ -642,21 +703,20 @@ class MultiplexerMetaClass(abc.ABCMeta):
                 if not issubclass(cmd, base.LmiBaseCommand):
                     raise errors.LmiCommandError(module_name, name,
                             'COMMANDS dictionary must be composed of'
-                            ' LmiCommandBase subclasses, failed class: "%s"'
+                            ' LmiBaseCommand subclasses, failed class: "%s"'
                             % cmd.__name__)
-                if not cmd.is_end_point():
+                if not cmd.is_end_point() and not cmd.has_own_usage():
+                    LOG().warn('Command "%s.%s" is missing usage string.'
+                            ' It will be inherited from parent command.',
+                            cmd.__module__, cmd.__name__)
                     cmd.__doc__ = dcl['__doc__']
             def _new_child_commands(_cls):
                 """ Returns list of subcommands. """
                 return cmds
             dcl['child_commands'] = classmethod(_new_child_commands)
 
-            # check documentation
-            if dcl.get('__doc__', None) is None:
-                LOG().warn('Command "%s.%s" is missing description string.',
-                    dcl['__module__'], name)
-
             _handle_usage(name, dcl)
             _handle_fallback_command(name, bases, dcl)
+            _handle_format_options(name, bases, dcl)
 
         return super(MultiplexerMetaClass, mcs).__new__(mcs, name, bases, dcl)
