@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -117,7 +117,7 @@ class LmiCommandMultiplexer(base.LmiBaseCommand):
             cmd = cmd_cls(self.app, cmd_name, self)
         except KeyError:
             self.app.stderr.write(self.get_usage())
-            LOG().critical('unexpected command "%s"', cmd_name)
+            LOG().critical('Unexpected command "%s".', cmd_name)
             return 1
         return cmd.run(args)
 
@@ -130,18 +130,22 @@ class LmiCommandMultiplexer(base.LmiBaseCommand):
         """
         if not isinstance(args, (list, tuple)):
             raise TypeError("args must be a list")
-        full_args = self.cmd_name_args[1:] + args
-        # check the --help ourselves (the default docopt behaviour checks
-        # also for --version)
+        full_args = self.get_cmd_name_parts(for_docopt=True) + args
         docopt_kwargs = {
-                'help' : False,
-                # let's ignore options following first command for generated
-                # usage string
-                'options_first' : not self.has_own_usage()
+            # check the --help ourselves (the default docopt behaviour checks
+            # also for --version)
+            'help' : False,
+            # let's ignore options following first command for generated
+            # usage string and when a height of this branch is > 2
+            'options_first' : not self.has_own_usage()
+                    or any(   not cmd.is_end_point()
+                          for cmd in self.child_commands().values()
+                          if  not args or args[0] not in self.child_commands()
+                              or self.child_commands()[args[0]] is cmd)
         }
         options = docopt(self.get_usage(), full_args, **docopt_kwargs)
-        if options.pop('--help', False):
-            self.app.stdout.write(self.get_usage())
+        if options.pop('--help', False) or (args and args[0] == '--help'):
+            self.app.stdout.write(self.get_usage(proper=True))
             return 0
         if (   self.fallback_command() is not None
            and (not args or args[0] not in self.child_commands())):
