@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -267,33 +267,45 @@ class TableFormatter(ListFormatter):
         super(TableFormatter, self).__init__(stream, padding, no_headings)
         self.stash = []
 
-    def print_text_row(self, row, column_size):
+    def print_text_row(self, row, column_sizes):
         for i in xrange(len(row)):
-            size = column_size[i]
-            item = self.render_value(row[i])
+            size = column_sizes[i]
+            # Convert to unicode to compute correct length of utf-8 strings
+            # (e.g. with fancy trees with utf-8 graphics).
+            item = (    unicode(row[i]) if not isinstance(row[i], str)
+                   else row[i].decode(self.encoding))
             if i < len(row) - 1:
                 item = item.ljust(size)
-            self.out.write(item)
-            self.out.write(" ")
+            self.out.write(self.render_value(item))
+            if i < len(row) - 1:
+                self.out.write(" ")
         self.out.write("\n")
         self.line_counter += 1
 
     def print_stash(self):
-        if not self.stash:
-            return
-
         # Compute column sizes
         column_sizes = []
-        for i in xrange(len(self.column_names)):
-            column_sizes.append(len(self.column_names[i]))
-        for row in self.stash:
+        rows = iter(self.stash)
+        try:
+            if self.column_names is None:
+                row = rows.next()
+            else:
+                row = self.column_names
             for i in xrange(len(row)):
-                row_length = len(unicode(row[i]))
+                column_sizes.append(len(row[i]))
+        except StopIteration:
+            pass # empty stash
+
+        for row in rows:
+            for i in xrange(len(row)):
+                row_length = len(
+                         unicode(row[i]) if not isinstance(row[i], str)
+                    else row[i].decode(self.encoding))
                 if column_sizes[i] < row_length:
                     column_sizes[i] = row_length
 
         # print headers
-        if not self.no_headings:
+        if not self.no_headings and self.column_names:
             self.print_text_row(self.column_names, column_sizes)
         # print stashed rows
         for row in self.stash:
