@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -94,10 +94,10 @@ class Session(object):
                     yield connection
                     successful_connections += 1
             except Exception as exc:
-                LOG().error('failed to make a connection to "%s": %s',
+                LOG().error('Failed to make a connection to "%s": %s',
                         hostname, exc)
         if successful_connections == 0:
-            raise errors.LmiNoConnections('no successful connection made')
+            raise errors.LmiNoConnections('No successful connection made.')
 
     def _connect(self, hostname, interactive=False):
         """
@@ -118,14 +118,12 @@ class Session(object):
             kwargs['prompt_prefix'] = '[%s] ' % hostname
         connection = connect(hostname, username, password, **kwargs)
         if connection is not None:
-            LOG().debug('connection to host "%s" successfully created',
-                    hostname)
             tp = connection._client._cliconn.creds
             if tp is None:
                 tp = ('', '')
             self._credentials[hostname] = (tp[0], tp[1], True)
         else:
-            LOG().error('failed to connect to host "%s"', hostname)
+            LOG().error('Failed to connect to host "%s".', hostname)
         return connection
 
     @property
